

import collections
import traceback
import json
import sys


from .StackTraceItem import StackTraceItem
from .ExceptionObject import ExceptionObject





"""
def dump(obj, name=None, indent=""):
	if name:
		s = name + "="
	else:
		s = ""

	if obj is None:
		print(indent + s + "(none)")
		return

	if callable(obj):
		#print(indent + s + "(callable)")
		return

	if name in [ "f_builtins" ]:
		return

	if isinstance(obj, (int, str, bool, list, tuple, dict)):
		print(indent + s + repr(obj))
		return

	print(indent + s + obj.__class__.__name__)
	for p in dir(obj):
		if p in [ "__call__", "__class__" ]:
			continue
		dump(getattr(obj, p), p, indent + "\t")
#
"""


"""
#
# Processes the exception.
#
# @param	Exception exception		The exception to parse
#
def getExceptionData(exception:Exception):
	assert isinstance(exception, Exception)

	exceptionClassName = exception.__class__.__name__

	exceptionLines = []
	for line in str(exception).splitlines():
		line = line.strip()
		if len(line) > 0:
			exceptionLines.append(line)
	exceptionTextHR = " ".join(exceptionLines)
	if not exceptionTextHR:
		exceptionTextHR = None

	stackTrace = []
	stackTraceList = traceback.extract_stack(exception.__traceback__.tb_frame)
	for stackTraceComponent in stackTraceList:
		stackTrace.append(StackTraceItem(
			stackTraceComponent.filename,
			stackTraceComponent.lineno,
			stackTraceComponent.name,
			stackTraceComponent.line
			))

	return exceptionClassName, exceptionTextHR, stackTrace, exception.__context__
#
"""



def _analyseNestedException(exception) -> ExceptionObject:
	exceptionLines = []
	for line in str(exception).splitlines():
		line = line.strip()
		if len(line) > 0:
			exceptionLines.append(line)
	exceptionTextHR = " ".join(exceptionLines)
	if not exceptionTextHR:
		exceptionTextHR = None

	if exception.__context__:
		nestedException = _analyseNestedException(exception.__context__)
	else:
		nestedException = None


	return ExceptionObject(exception, exception.__class__.__name__, exceptionTextHR, None, nestedException)
#




def analyseException(exception, ignoreJKTypingCheckFunctionSignatureFrames:bool = False, ignoreJKTestingAssertFrames:bool = False) -> ExceptionObject:
	assert isinstance(ignoreJKTypingCheckFunctionSignatureFrames, bool)
	assert isinstance(ignoreJKTestingAssertFrames, bool)

	_type, _exception, _traceback = sys.exc_info()

	if _exception is None:
		_exception = exception
		_type = type(exception)

	exceptionLines = []
	for line in str(_exception).splitlines():
		line = line.strip()
		if len(line) > 0:
			exceptionLines.append(line)
	exceptionTextHR = " ".join(exceptionLines)
	if not exceptionTextHR:
		exceptionTextHR = None

	stackTrace = []
	#print("-X-------------------------------")
	#print("Type =", _type)
	#print("Value =", repr(str(_exception)))
	##print("Trace =")
	##print("Traceback=", _traceback)
	#print("Trace =", traceback.extract_tb(_traceback))
	for stElement in traceback.extract_tb(_traceback):
		#assert isinstance(stElement, traceback.FrameSummary)
		# print(">", repr(stElement.filename), "|", repr(stElement.name))
		if ignoreJKTypingCheckFunctionSignatureFrames:
			if (stElement.name == "wrapped") \
				and (
					(stElement.filename.find("jk_typing/checkFunctionSignature.py") >= 0) or
					(stElement.filename.find("jk_typing\\checkFunctionSignature.py") >= 0)
				):
				continue
		if ignoreJKTestingAssertFrames:
			if (
					(stElement.filename.find("jk_testing/Assert.py") >= 0) or
					(stElement.filename.find("jk_testing\\Assert.py") >= 0)
				):
				continue
		stackTrace.append(StackTraceItem(
			stElement.filename,
			stElement.lineno,
			stElement.name,
			stElement.line
			))
	#print("-X-------------------------------")

	if _exception.__context__:
		nestedException = _analyseNestedException(_exception.__context__)
	else:
		nestedException = None


	return ExceptionObject(_exception, _type.__name__, exceptionTextHR, stackTrace, nestedException)
#












from .StackTraceItem import StackTraceItem




class ExceptionObject(object):

	################################################################################################################################
	## Constructor
	################################################################################################################################

	def __init__(self, exceptionClass, exceptionClassName:str, exceptionTextHR:str, stackTrace:list, nestedException):
		self.exceptionClass = exceptionClass

		self.exceptionClassName = exceptionClassName

		if exceptionTextHR is not None:
			assert isinstance(exceptionTextHR, str)
		self.exceptionTextHR = exceptionTextHR

		if stackTrace is not None:
			assert isinstance(stackTrace, (list, tuple))
			for item in stackTrace:
				assert isinstance(item, StackTraceItem)
			self.stackTrace = stackTrace
		else:
			self.stackTrace = None

		if nestedException is not None:
			assert isinstance(nestedException, ExceptionObject)
		self.nestedException = nestedException
	#

	################################################################################################################################
	## Public Properties
	################################################################################################################################

	################################################################################################################################
	## Helper Methods
	################################################################################################################################

	################################################################################################################################
	## Public Methods
	################################################################################################################################

	def toStrList(self):
		outStrList = []

		e = self
		indent = ""
		while True:
			outStrList.append(indent + e.exceptionClassName)
			if e.exceptionTextHR is not None:
				outStrList.append(indent + ": exceptionTextHR:")
				outStrList.append(indent + ":\t" + e.exceptionTextHR)
			if e.stackTrace is not None:
				outStrList.append(indent + ": stackTrace:")
				for item in reversed(e.stackTrace):
					outStrList.append(indent + ":\t" + str(item))
			if e.nestedException:
				e = e.nestedException
				outStrList.append(indent + "\- nestedException:")
				indent += "\t"
			else:
				outStrList.append(indent + "\-")
				break

		return outStrList
	#

	def dump(self):
		e = self
		indent = ""
		while True:
			print(indent + e.exceptionClassName)
			if e.exceptionTextHR is not None:
				print(indent + ": exceptionTextHR:")
				print(indent + ":\t" + e.exceptionTextHR)
			if e.stackTrace is not None:
				print(indent + ": stackTrace:")
				for item in reversed(e.stackTrace):
					print(indent + ":\t" + str(item))
			if e.nestedException:
				e = e.nestedException
				print(indent + "\- nestedException:")
				indent += "\t"
			else:
				print(indent + "\-")
				break
	#

	#
	# Convert this object to JSON (recursively).
	#
	# @param	bool bRecursive		If `True` (which is the default) nested exceptions are serialized as well. If `False` these get skipped.
	#
	def toJSON(self, bRecursive:bool = True) -> dict:
		ret = {
			"text": self.exceptionTextHR,
			"exception": self.exceptionClassName,
		}
		if self.stackTrace is not None:
			ret["stacktrace"] = [ x.toJSON() for x in self.stackTrace ]
		if bRecursive and self.nestedException:
			ret["nested"] = self.nestedException.toJSON()
		else:
			ret["stacktrace"] = None
		return ret
	#

	#
	# Convert this object to JSON (only this level).
	#
	def toJSON_flat(self):
		ret = {
			"text": self.exceptionTextHR,
			"exception": self.exceptionClassName,
		}
		if self.stackTrace is not None:
			ret["stacktrace"] = [ x.toJSON() for x in self.stackTrace ]
		else:
			ret["stacktrace"] = None
		return ret
	#

	@staticmethod
	def fromJSON(data:dict):
		return ExceptionObject(
			None,
			data["exception"],
			data["text"],
			([ StackTraceItem.fromJSON(x) for x in data["stacktrace"] ]) if data.get("stacktrace") else None,
			ExceptionObject.fromJSON(data["nested"]) if data.get("nested") else None
		)
	#

#















################################################################################
################################################################################
###
###  This file is automatically generated. Do not change this file! Changes
###  will get overwritten! Change the source file for "setup.py" instead.
###  This is either 'packageinfo.json' or 'packageinfo.jsonc'
###
################################################################################
################################################################################


from setuptools import setup

def readme():
	with open("README.md", "r", encoding="UTF-8-sig") as f:
		return f.read()

setup(
	author = "Jürgen Knauth",
	author_email = "pubsrc@binary-overflow.de",
	classifiers = [
		"Development Status :: 5 - Production/Stable",
		"License :: OSI Approved :: Apache Software License",
		"Programming Language :: Python :: 3",
		"Topic :: Software Development :: Testing",
	],
	description = "As the python exception API is quite a bit obscure this python module wraps around python exceptions to provide a clean interface for analysis and logging purposes.",
	include_package_data = False,
	install_requires = [
	],
	keywords = [
		"exception handling",
		"debugging",
	],
	license = "Apache2",
	name = "jk_exceptionhelper",
	packages = [
		"jk_exceptionhelper",
	],
	version = "0.2021.12.14",
	zip_safe = False,
	long_description = readme(),
	long_description_content_type="text/markdown",
)





# import collections

# StackTraceItem = collections.namedtuple("StackTraceItem", [
#	"filePath", "lineNo", "callingScope", "sourceCodeLine"
# ])


class StackTraceItem(object):
	
	def __init__(self, filePath:str, lineNo:int, callingScope:str, sourceCodeLine:str):
		assert isinstance(filePath, str)
		assert isinstance(lineNo, int)
		assert isinstance(callingScope, str)
		assert isinstance(sourceCodeLine, str)

		self.filePath = filePath
		self.lineNo = lineNo
		self.callingScope = callingScope
		self.sourceCodeLine = sourceCodeLine
	#

	def __str__(self):
		# return "(" + (self.callingScope if self.callingScope is not None else "(none)") + ", " + self.filePath + ":" + str(self.lineNo) + ", " + repr(self.sourceCodeLine) + ")"
		return "(" + self.filePath + ":" + str(self.lineNo) + ", " + repr(self.sourceCodeLine) + ")"
	#

	def __repr__(self):
		# return "(" + (self.callingScope if self.callingScope is not None else "(none)") + ", " + self.filePath + ":" + str(self.lineNo) + ", " + repr(self.sourceCodeLine) + ")"
		return "(" + self.filePath + ":" + str(self.lineNo) + ", " + repr(self.sourceCodeLine) + ")"
	#

	def toJSON(self) -> dict:
		return {
			"file": self.filePath,
			"line": self.lineNo,
			# "callingscope": self.callingScope,
			"sourcecode": self.sourceCodeLine,
		}
	#

	@staticmethod
	def fromJSON(data:dict):
		return StackTraceItem(
			data["file"],
			data["line"],
			# data["callingscope"],
			data["sourcecode"]
		)
	#

#


